#include "RotationQuery.h"

RotationQuery::RotationQuery(TextCollection *tc, OutputWriter &ow, bool vrb, 
                             bool incr, bool rvrs, unsigned rprt)
    : Query(tc, ow, vrb, incr, rvrs, rprt), psize(0)
{  
}


void RotationQuery::firstStep(void) 
{
    if (patlen != tc->getRotationLength())
    {
        std::cerr << "readaligner: rotation index was built for reads of length " << tc->getRotationLength() << ":" << std::endl
                  << "unable to search for pattern \"" << p->getName() << "\" of length " << patlen << std::endl;
        std::abort();
    }

    psize = patlen / (klimit + 1);
    counterstart = callcounter;
    for (unsigned i = 1; i <= patlen; i++) {
	if (i % psize == 0) {
	    if (debug) std::cerr << "new try: " << i << "\n";
	    nextStep(i, 0, 0, -1);
	}
    }
}

void RotationQuery::exactMatch(int pos, unsigned cnt, int k, int maxk)
{
    callcounter++;
//    cout << "exact matching: " << pos << " " << cnt << " " << k << " " << maxk << endl;
    int i = pos - 1;
    if (!pushChar(pat[i]))
        return;
    while (i % psize != 0 && pushChar(pat[i-1]))
        --i;

    cnt += pos - i;
    if (i % psize == 0) 
        nextStep(i, cnt, k, maxk);
    
    // Rewind stack
    while (i != pos) {
        ++i;
	popChar();
    }
}

void RotationQuery::nextStep(int pos, unsigned cnt, int k, int maxk) {
    callcounter++;
    if (callcounter - counterstart > rlimit) return;
    if (sum >= report) return;

    if (pos % psize == 0) maxk++;

    bool newround = false; // True if '#' is in stack
    if (pos == 0) {
	if (!pushChar('#')) return;
	newround = true; // pushed '#'
	pos += patlen;
    }

    if (cnt == patlen) {
        newMatch(pos); 
        if (newround) popChar(); // pop '#'
        return; 
    }

    if (k == maxk)
    {
        exactMatch(pos, cnt, k, maxk);
        if (newround) popChar(); // pop '#'
        return;
    }
//    std::cout << pos << " " << cnt << " " << k << " " << maxk << "\n";
    // ALPHABET has been defined in BTSearch
    for (const char *c = ALPHABET; c < ALPHABET + ALPHABET_SIZE; ++c) {
	if (!pushChar(*c)) continue;
	int newk = k;
	if (pat[pos - 1] != *c) newk = k + 1;
	if (newk <= maxk) nextStep(pos - 1, cnt + 1, newk, maxk);
	popChar();
    }
    if (newround) popChar(); // pop '#'
}
